/*
*	Appbrowser-cli
*	v0.5
*
*	Updated search for >= 4.4
*
*	v0.4i: Updated search for >= 2.9
*	v0.4h: Removed install option, added argv[1] search
*	v0.4g: Fixes, pgup, pgdown, long names.
*
*
*	(C) Curaga
*	Under the GPLv3
*
*	Contains some code from Appbrowser, (C) Robert Shingledecker
*/

#include <ncurses.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>

#define SEARCH_STR 35
#define MAX_EXTN_NAME 1024
#define OPTN_MAX 60
#define separate_extension for(j=0;j<MAX_EXTN_NAME;j++) { \
			if(extn[extnpick][j]!='\n') tmppath[j]=extn[extnpick][j]; \
			else { tmppath[j]='\0'; break; } }
#define connerr() quitit("Connection error.\n",1)

static WINDOW *left, *right, *info, *windows[3], *focus, *lefty, *righty;
static int downloadonly=0, extnlines=0, results=0, rightlines=0, halfw=0;
static char tmpdir[]="/tmp/appbrowserXXXXXX", tmppath[PATH_MAX], *infolstram, *rightram;
static char **extn, **pr_extn, **rightline, *pr_infolstram;
static FILE *infolst;

static void quitit(char *moi, int value){
	endwin();
	fprintf(stderr,"%s\n",moi);

	chdir("/");
	snprintf(tmppath,PATH_MAX,"rm -rf %s",tmpdir);
	system(tmppath);

	exit(value);
}

static void wgetlist(){
	results=system("/usr/bin/tce-fetch.sh info.lst tcz 2>/dev/null");

//results=system("cp /home/tc/c/appbrowser-cli/info.lst /tmp/appbro*");

	if(results) connerr();
}

static void dosearch(const char *what) {

	unlink("info.lst");

	snprintf(tmppath,PATH_MAX,"/usr/bin/search.sh %s 2>/dev/null > info2", what);
	results=system(tmppath);
	if(results) connerr();

	// If info.lst does not exist, use info2
	if (access("info.lst", F_OK))
		rename("info2", "info.lst");
}

static void focuswindow(){
	wattron(focus,A_BOLD|COLOR_PAIR(1));
	box(focus,0,0);
	wrefresh(focus);
	wattroff(focus,A_BOLD|COLOR_PAIR(1));
	if (focus==lefty) { redrawwin(left); wrefresh(left); }
	else if (focus==righty) { redrawwin(right); wrefresh(right); }
}

static void unfocuswindow(WINDOW *old){
	wattron(old,COLOR_PAIR(1));
	box(old,0,0);
	wrefresh(old);
	wattroff(old,COLOR_PAIR(1));
	if (old==lefty) { redrawwin(left); wrefresh(left); }
	else if (old==righty) { redrawwin(right); wrefresh(right); }
}

static void initwindows(){
	int i;

	for(i=0;i<3;i++){
		wclear(windows[i]);
		wattron(windows[i],COLOR_PAIR(1));
		box(windows[i],0,0);
		if (i < 2) wrefresh(windows[i]);
		wattroff(windows[i],COLOR_PAIR(1));
	}
}

static void readinfolst(){
	struct stat sb;
	int i=1,j=0;

	if(infolstram) {
		free(infolstram);
		infolstram=NULL;
	}
	if(extn) {
		free(extn);
		extn=NULL;
	}
	if(pr_infolstram) {
		free(infolstram);
		infolstram=NULL;
	}
	if(pr_extn) {
		free(extn);
		extn=NULL;
	}

	snprintf(tmppath,PATH_MAX,"busybox cut -c 0-%d info.lst > info.new",halfw-3);
	system(tmppath);

	infolst=fopen("info.lst","r");
	stat("info.lst",&sb);
	infolstram=(char *) calloc((size_t) sb.st_size,sizeof(char));
	if(infolstram==NULL) goto quit;
	fread(infolstram,sizeof(char),(size_t) sb.st_size-1,infolst); // EOF is a character too
	fclose(infolst);

	infolst=fopen("info.new","r");
	stat("info.new",&sb);
	pr_infolstram=(char *) calloc((size_t) sb.st_size,sizeof(char));
	if(pr_infolstram==NULL) goto quit;
	fread(pr_infolstram,sizeof(char),(size_t) sb.st_size-1,infolst); // EOF is a character too
	fclose(infolst);

	infolst=popen("wc -l info.lst","r");
	if( fscanf(infolst,"%d",&extnlines)!=1 ) quitit("",1);
	pclose(infolst);

	extn=(char **) calloc((size_t)extnlines,sizeof(char *));
	if(extn==NULL) goto quit;
	extn[0]=infolstram;
	for(i=1;i<extnlines;i++){
		for(j=0;j<MAX_EXTN_NAME;j++){
			if(extn[i-1][j]=='\n') {
				extn[i]=&extn[i-1][j+1];
				break;
			}
		}
	}

	pr_extn=(char **) calloc((size_t)extnlines,sizeof(char *));
	if(pr_extn==NULL) goto quit;
	pr_extn[0]=pr_infolstram;
	for(i=1;i<extnlines;i++){
		for(j=0;j<MAX_EXTN_NAME;j++){
			if(pr_extn[i-1][j]=='\n') {
				pr_extn[i]=&pr_extn[i-1][j+1];
				break;
			}
		}
	}

	wclear(left);
	mvwprintw(left,0,0,"%s",pr_infolstram);
	wrefresh(left);

	return;

	quit:
	quitit("",1);
}

static void readright(const char *filu){
	struct stat sb;
	int i=1,j=0;
	char *filu2;

	if(rightram) {
		free(rightram);
		rightram=NULL;
	}
	if(rightline) {
		free(rightline);
		rightline=NULL;
	}

	filu2=strdup(filu);
	snprintf(tmppath,PATH_MAX,"/usr/bin/tce-fetch.sh %s 2>/dev/null",filu2);

	results=system(tmppath);
	if(results) {
		wprintw(right,"Error - no such file or file can't be read");
		wrefresh(right);
		return;
	}

	infolst=fopen(filu2,"r");
	stat(filu2,&sb);
	rightram=(char *) calloc((size_t) sb.st_size,sizeof(char));
	if(rightram==NULL) goto quit;
	fread(rightram,sizeof(char),sb.st_size-1,infolst); // EOF is a character too
	fclose(infolst);

	snprintf(tmppath,PATH_MAX,"wc -l %s",filu2);
	infolst=popen(tmppath,"r");
	if( fscanf(infolst,"%d",&rightlines)!=1 ) {
		wprintw(right,"Error - no such file or file can't be read");
		return;
	}
	pclose(infolst);

	rightline=(char **) calloc((size_t) rightlines,sizeof(char *));
	if(rightline==NULL) goto quit;
	rightline[0]=rightram;
	for(i=1;i<rightlines;i++){
		for(j=0;j<MAX_EXTN_NAME;j++){
			if(rightline[i-1][j]=='\n') {
				rightline[i]=&rightline[i-1][j+1];
				break;
			}
		}
	}

	wclear(right);
	mvwprintw(right,0,0,"%s",rightram);
	wrefresh(right);

	free(filu2);
	return;

	quit:
	quitit("",1);
}

static void wprintcenter(WINDOW *win, const int line, int width,
const char *msg) {
	if (width==0) width=COLS;
	mvwprintw(win,line,(int)(width-strlen(msg))/2,"%s",msg);
}

int main(int argc, char **argv){

int i=0,j=0,infoh=0,extnpick=0,top=0,hilight=0,rtop=0;
char *infomsg="(ESC Q)uit (S)earch (O)ptions (G)et extension (F)iles (D)ependencies";
char optnmsg[OPTN_MAX]="Current options - download only: no";
char *empty="                                                                      ";
char searchmsg[SEARCH_STR],tmppath2[MAX_EXTN_NAME];

mkdtemp(tmpdir);
chdir(tmpdir); // we work in a secure temporary directory.
initscr();
start_color();
cbreak();
noecho();
keypad(stdscr,TRUE);
curs_set(0);
wprintcenter(stdscr,LINES-1,COLS,infomsg);
wprintcenter(stdscr,LINES-2,COLS,optnmsg);
mvaddch(LINES-1,0,ACS_DIAMOND);
mvaddch(LINES-1,COLS-1,ACS_DIAMOND);
refresh();

init_pair(1,COLOR_GREEN, COLOR_BLACK);
init_pair(2,COLOR_WHITE, COLOR_BLUE);
infoh = LINES/3;
halfw = COLS%2 ? (COLS-1)/2 : COLS/2;

left = newwin(LINES-4,halfw-2,1,1);
right = newwin(LINES-4,halfw-2,1,halfw+1);
lefty = newwin(LINES-2,halfw,0,0);
righty = newwin(LINES-2,halfw,0,halfw);
info = newwin(infoh,halfw,infoh,COLS/4);
windows[0]=lefty;
windows[1]=righty;
windows[2]=info;
focus=lefty;
initwindows();
focuswindow();

// All wgetty

if (argc > 1) {
	dosearch(argv[1]);
} else {
	wgetlist();
}
readinfolst();

// Draw in window

mvwchgat(left,extnpick,0,-1,A_NORMAL,2,NULL);
wrefresh(left);

// Main event loop

while (1) {
i=getch();
switch (i){
	case 'g':
	case 'G':
		wclear(right);
		separate_extension
		wprintw(right,"Getting extension %s\n\n",tmppath);
		wrefresh(right);

		strncpy(tmppath2,tmppath,MAX_EXTN_NAME);
		if(tmppath2[MAX_EXTN_NAME-1]!='\0') tmppath2[MAX_EXTN_NAME-1]='\0';

		if(downloadonly) snprintf(tmppath,PATH_MAX,"tce-load -w %s",tmppath2);
		else snprintf(tmppath,PATH_MAX,"tce-load -w -i %s",tmppath2);

		def_prog_mode();
		endwin();

		results=system(tmppath);

		reset_prog_mode();
		curs_set(0);
		refresh();

		if(!results) wprintw(right,"Installed/Downloaded OK");
		else {
			wprintw(right,"Error:\n\n");
			infolst=fopen("/tmp/aberr","r");
			while(fgets(tmppath,PATH_MAX,infolst)!=NULL) {
				wprintw(right,"%s",tmppath);
			}
			fclose(infolst);
		}
		wrefresh(right);
	break;
	case 10: // enter
		wclear(right);
		separate_extension
		wprintw(right,"Loading info for extension %s\n\n",tmppath);
		wrefresh(right);

		strcat(tmppath,".info");
		readright(tmppath);
	break;
	case KEY_NPAGE: //PgDown
		if(focus==lefty) {
		  if(extnpick < (extnlines-LINES+4)) {
			mvwchgat(left,hilight,0,-1,A_NORMAL,0,NULL);

			extnpick+=LINES-4;

			wclear(left);
			top+=LINES-4;
			mvwprintw(left,0,0,"%s",pr_extn[top]);
			mvwchgat(left,hilight,0,-1,A_NORMAL,2,NULL);

			wrefresh(left);
		  } else {
			mvwchgat(left,hilight,0,-1,A_NORMAL,0,NULL);

			extnpick=extnlines-1;

			wclear(left);
			top=extnlines-LINES+4;
			hilight=LINES-5;
			mvwprintw(left,0,0,"%s",pr_extn[top]);
			mvwchgat(left,hilight,0,-1,A_NORMAL,2,NULL);

			wrefresh(left);
		  }
		}
	break;
	case KEY_PPAGE: //PgUp
		if(focus==lefty) {
		  if(top > (LINES-5)) {
			mvwchgat(left,hilight,0,-1,A_NORMAL,0,NULL);

			extnpick-=LINES-4;

			wclear(left);
			top-=LINES-4;
			mvwprintw(left,0,0,"%s",pr_extn[top]);
			mvwchgat(left,hilight,0,-1,A_NORMAL,2,NULL);

			wrefresh(left);
		  } else {
			mvwchgat(left,hilight,0,-1,A_NORMAL,0,NULL);

			extnpick=top=hilight=0;

			wclear(left);
			mvwprintw(left,0,0,"%s",pr_extn[top]);
			mvwchgat(left,hilight,0,-1,A_NORMAL,2,NULL);

			wrefresh(left);
		  }
		}
	break;
	case 'f':
	case 'F':
		wclear(right);
		separate_extension
		wprintw(right,"Loading file list for extension %s\n\n",tmppath);
		wrefresh(right);

		strcat(tmppath,".list");
		readright(tmppath);
	break;
	case 'd':
	case 'D':
		wclear(right);
		separate_extension
		wprintw(right,"Loading dependency info for extension %s\n\n",tmppath);
		wrefresh(right);

		strcat(tmppath,".dep");
		readright(tmppath);
	break;
	case KEY_UP:
		if(focus==lefty && extnpick > 0) {
			mvwchgat(left,hilight,0,-1,A_NORMAL,0,NULL);
			extnpick--;
			if(hilight<=3 && top){
				wclear(left);
				top--;
				mvwprintw(left,0,0,"%s",pr_extn[top]);
				mvwchgat(left,hilight,0,-1,A_NORMAL,2,NULL);
			} else {
				hilight--;
				mvwchgat(left,hilight,0,-1,A_NORMAL,2,NULL);
			}
		 wrefresh(left);
		 } else if(focus==righty && rtop && rightline) {
		 	wclear(right);
			rtop--;
			mvwprintw(right,0,0,"%s",rightline[rtop]);
			wrefresh(right);
		 }
	break;
	case KEY_DOWN:
		if(focus==lefty && extnpick < (extnlines-1)) {
			mvwchgat(left,hilight,0,-1,A_NORMAL,0,NULL);
			extnpick++;
			if(hilight>=(LINES-8)){
				wclear(left);
				top++;
				mvwprintw(left,0,0,"%s",pr_extn[top]);
				mvwchgat(left,hilight,0,-1,A_NORMAL,2,NULL);
			} else {
				hilight++;
				mvwchgat(left,hilight,0,-1,A_NORMAL,2,NULL);
			}
		 wrefresh(left);
		 } else if(focus==righty && rtop < (rightlines-1) && rightline) {
		 	wclear(right);
			rtop++;
			mvwprintw(right,0,0,"%s",rightline[rtop]);
			wrefresh(right);
		 }
	break;
	case 'q':
	case 'Q':
	case 27:
		goto quit;
	case 9:
		if (focus==lefty) {
			focus=righty;
			unfocuswindow(lefty);
		}
		else {
			focus=lefty;
			unfocuswindow(righty);
		}
		focuswindow();
	break;
	case 's':
	case 'S':
		// Search window
		j=0;
		wclear(info);
		unfocuswindow(lefty);
		unfocuswindow(righty);
		focus=info;
		focuswindow();
		curs_set(1);

		wattron(info,COLOR_PAIR(2));
		wprintcenter(info,infoh-2,halfw,"   OK   ");
		wattroff(info,COLOR_PAIR(2));
		wprintcenter(info,1,halfw,"Search:");
		wprintcenter(info,5,halfw,"An empty search reloads the list");
		wattron(info,COLOR_PAIR(2));
		for(i=3;i<(halfw-3);i++) mvwaddch(info,3,i,' ');
		wmove(info,3,3);
		wrefresh(info);
		echo();

		wgetnstr(info,searchmsg,SEARCH_STR-1);

		noecho();
		curs_set(0);

		focus=lefty;
		unfocuswindow(righty);
		focuswindow();

		if(searchmsg[0]=='\0'){
			wgetlist();
		} else {
			dosearch(searchmsg);
		}
		readinfolst();
		hilight=extnpick=top=0;
		mvwchgat(left,hilight,0,-1,A_NORMAL,2,NULL);
		wrefresh(left);

		// End search window
	break;
	case 'o':
	case 'O':
		// Options window
		i=j=0;
		wclear(info);
		unfocuswindow(lefty);
		unfocuswindow(righty);
		focus=info;
		focuswindow();

		wprintcenter(info,1,halfw,"Options");
		mvwprintw(info,3,2,"Download only:");
		wattron(info,COLOR_PAIR(2));
		wprintcenter(info,infoh-2,halfw,"   OK   ");
		wattroff(info,COLOR_PAIR(2));

		while (1) {
		switch (j){
			case KEY_RIGHT:
			case KEY_LEFT:
			case ' ':
				if (i==0) {
					if (downloadonly==0) downloadonly=1;
					else downloadonly=0;
				}
			break;
			case KEY_UP:
				i--;
				if (i<0) i=0;
			break;
			case KEY_DOWN:
				i++;
				if (i>1) i=1;
			break;
			case 10:
				focus=lefty;
				unfocuswindow(righty);
				focuswindow();
			break;
		}
		if (j==10) break;
			wattron(info,COLOR_PAIR(2));
			(downloadonly ? mvwprintw(info,3,halfw-6,"Yes") : mvwprintw(info,3,halfw-6,"No "));
			wattroff(info,COLOR_PAIR(2));

			if (i==0) {
			mvwchgat(info,3,halfw-7,5,A_BOLD,2,NULL);
			mvwchgat(info,infoh-2,(halfw-8)/2,8,A_NORMAL,2,NULL);
			}
			else {
			mvwchgat(info,3,halfw-7,5,A_NORMAL,2,NULL);
			mvwchgat(info,infoh-2,(halfw-8)/2,8,A_BOLD,2,NULL);
			}
			wrefresh(info);

		j=getch();
		}
		snprintf(optnmsg,OPTN_MAX,"Current options - download only: %s",(downloadonly?"yes":"no"));
		wprintcenter(stdscr,LINES-2,COLS,empty);
		wprintcenter(stdscr,LINES-2,COLS,optnmsg);
		refresh();
		mvwchgat(left,hilight,0,-1,A_NORMAL,2,NULL);
		wrefresh(left);
		// End options window
	break;
}
}

quit:
quitit("",0);
return 0;
}
